import { setBlockType } from 'prosemirror-commands';
import { textblockTypeInputRule } from 'prosemirror-inputrules';
import { Fragment, Slice } from 'prosemirror-model';
import { Plugin } from 'prosemirror-state';
import { hasParentNodeOfType } from 'prosemirror-utils';
import { resetCodeblock } from './commands';
import { cbAction, cbType, codeBlock, langAttr } from './const';
import { handlePaste } from './handle-paste';
export { resetCodeblock } from './commands';
export { codeBlock as codeBlockNodeName, langAttr as codeBlockLangAttr, cbType as codeBlockType, } from './const';
export const CodeBlock = (builder, opts) => {
    builder.addNode(codeBlock, () => ({
        spec: {
            attrs: { [langAttr]: { default: 'text' } },
            content: 'text*',
            group: 'block',
            code: true,
            marks: '',
            selectable: true,
            allowSelection: true,
            parseDOM: [
                {
                    tag: 'pre',
                    preserveWhitespace: 'full',
                    getAttrs: (node) => ({
                        [langAttr]: node.getAttribute(langAttr) || '',
                    }),
                },
            ],
            toDOM({ attrs }) {
                return ['pre', attrs[langAttr] ? attrs : {}, ['code', 0]];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: codeBlock,
                type: 'block',
                noCloseToken: true,
            },
        },
        toYfm: (state, node) => {
            state.write('```' + (node.attrs[langAttr] || '') + '\n');
            state.text(node.textContent, false);
            state.ensureNewLine();
            state.write('```');
            state.closeBlock(node);
        },
    }));
    builder.addNode('fence', () => ({
        //  we adding this node only for define specific 'fence' parser token,
        //  which parse fence md token to code_block node
        spec: {},
        fromYfm: {
            tokenSpec: {
                name: codeBlock,
                type: 'block',
                noCloseToken: true,
                getAttrs: (tok) => ({ [langAttr]: tok.info || '' }),
            },
        },
        toYfm: () => {
            throw new Error('Unexpected toYfm() call on fence node');
        },
    }));
    builder.addKeymap((deps) => {
        const { codeBlockKey } = opts;
        const bindings = { Backspace: resetCodeblock };
        if (codeBlockKey)
            bindings[codeBlockKey] = setBlockType(cbType(deps.schema));
        return bindings;
    });
    builder.addInputRules(({ schema }) => ({ rules: [codeBlockRule(cbType(schema))] }));
    builder.addAction(cbAction, ({ schema, serializer }) => {
        const cb = cbType(schema);
        const cmd = (state, dispatch) => {
            if (!setBlockType(cb)(state))
                return false;
            if (dispatch) {
                const markup = serializer.serialize(Fragment.from(state.selection.content().content));
                dispatch(state.tr.replaceSelectionWith(cb.createAndFill({}, markup ? state.schema.text(markup) : null)));
            }
            return true;
        };
        return {
            isActive: (state) => hasParentNodeOfType(cb)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
    builder.addPlugin(() => new Plugin({
        props: {
            handleDOMEvents: {
                paste(view, e) {
                    if (handlePaste(view, e, Slice.empty)) {
                        e.preventDefault();
                        return true;
                    }
                    return false;
                },
            },
        },
    }), builder.Priority.High);
};
// Given a code block node type, returns an input rule that turns a
// textblock starting with three backticks into a code block.
function codeBlockRule(nodeType) {
    return textblockTypeInputRule(/^```$/, nodeType);
}
